"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs-extra"));
const path = __importStar(require("path"));
const ts = __importStar(require("typescript"));
const project_scanner_1 = require("../project-scanner");
class ClassDeclarationResolver {
    static createClassDeclarations(scanningEntryPointPath) {
        const unfilteredFiles = project_scanner_1.ProjectScanner.scanProject(scanningEntryPointPath);
        const filteredFiles = project_scanner_1.ProjectScanner.filterFiles(unfilteredFiles, ".ts");
        const filePaths = filteredFiles.map(file => file.filePath);
        const program = ts.createProgram(filePaths, {
            incremental: true,
            module: ts.ModuleKind.CommonJS,
            target: ts.ScriptTarget.ES5
        });
        let output = [];
        const checker = program.getTypeChecker();
        for (const sourceFile of program.getSourceFiles()) {
            output = output.concat(this.serializeClassDeclarations(sourceFile, checker));
        }
        return output;
    }
    static createClassDeclarationFile(projectRootPath, sourceCodeDirectory, declarationFileDirectory, declarationFileName, outDirectoryPath) {
        if (!fs.pathExistsSync(declarationFileDirectory)) {
            fs.mkdirSync(declarationFileDirectory);
        }
        const declarations = this.createClassDeclarations(path.join(projectRootPath, sourceCodeDirectory));
        if (outDirectoryPath) {
            declarations.forEach(declaration => {
                declaration.compiledFilePath = ClassDeclarationResolver.createCompiledFilePath(declaration.sourceFilePath, sourceCodeDirectory, outDirectoryPath);
            });
        }
        fs.writeFileSync(path.resolve(declarationFileDirectory + "/" + declarationFileName), JSON.stringify(declarations, undefined, 2));
    }
    static importClassDeclarationFromFile(filePath) {
        const resolvedFilePath = path.resolve(filePath);
        if (fs.existsSync(resolvedFilePath)) {
            const fileData = fs.readFileSync(resolvedFilePath);
            return JSON.parse(fileData.toString());
        }
        throw new Error("No class declaration file found.");
    }
    static createCompiledFilePath(sourceFilePath, sourceCodeDirectory, outDirectory) {
        sourceFilePath =
            sourceFilePath.substr(0, sourceFilePath.lastIndexOf(".")) + ".js";
        return sourceFilePath.replace(sourceCodeDirectory, outDirectory);
    }
    static isNodeExported(node) {
        return (
        // tslint:disable-next-line:no-bitwise
        (ts.getCombinedNodeFlags(node) & ts.ModifierFlags.Export) !== 0 ||
            (!!node.parent && node.parent.kind === ts.SyntaxKind.SourceFile));
    }
    static isSourceFile(source) {
        return source.isDeclarationFile !== undefined;
    }
    static serializeClassDeclarations(source, checker) {
        const output = [];
        if (this.isSourceFile(source)) {
            if (!source.isDeclarationFile) {
                ts.forEachChild(source, node => {
                    const classInformation = this.serializeClassInformation(node, checker);
                    if (classInformation) {
                        output.push({
                            classInformation,
                            sourceFilePath: node.getSourceFile().fileName
                        });
                    }
                });
            }
        }
        else {
            ts.forEachChild(source, subNode => {
                const classInformation = this.serializeClassInformation(subNode, checker);
                if (classInformation) {
                    output.push({
                        classInformation,
                        sourceFilePath: subNode.getSourceFile().fileName
                    });
                }
            });
        }
        return output;
    }
    static serializeClassInformation(node, checker) {
        if (this.isNodeExported(node)) {
            if (ts.isClassDeclaration(node) && node.name) {
                const symbol = checker.getSymbolAtLocation(node.name);
                if (symbol) {
                    return this.serializeClass(symbol, checker);
                }
            }
            else if (ts.isModuleDeclaration(node)) {
                this.serializeClassDeclarations(node, checker);
            }
        }
        return;
    }
    static serializeClass(symbol, checker) {
        const constructorType = checker.getTypeOfSymbolAtLocation(symbol, symbol.valueDeclaration);
        return {
            className: symbol.getName(),
            constructors: this.serializeClassConstructor(constructorType, checker)
        };
    }
    static serializeClassConstructor(constructorType, checker) {
        return constructorType.getConstructSignatures().map(signature => {
            return {
                parameters: this.serializeClassConstructorParameter(signature, checker)
            };
        });
    }
    static serializeClassConstructorParameter(signature, checker) {
        return signature.getParameters().map(parameter => {
            return {
                name: parameter.getName(),
                type: checker.typeToString(checker.getTypeOfSymbolAtLocation(parameter, parameter.valueDeclaration))
            };
        });
    }
}
exports.ClassDeclarationResolver = ClassDeclarationResolver;
//# sourceMappingURL=classDeclarationResolver.js.map